/* 
   ABOUT ANTLR:
   [The BSD License]
	Copyright (c) 2010 Terence Parr
	All rights reserved.
	Redistribution and use in source and binary forms, with or without modification, are permitted provided that the 
	following conditions are met:

	    1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
	    2. Redistributions in binary form must reproduce the above copyright notice, this 
	      list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
	    3. Neither the name of the author nor the names of its contributors may be used to endorse or promote products 
	      derived from this software without specific prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
	SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
	THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

/* 
 * ABOUT GalaxyGrammar:
 * Created by Timo Hanisch 2010 
 * timohanisch@gmx.de
 * */


grammar Galaxy;
options {
    backtrack=true;
    memoize=true;
    k=2;
    output = AST;
    ASTLabelType=CommonTree;
}
tokens{
	NEGATION;
}

scope Symbols {
	Set types; 
}

@header {
	package de.timoh.galaxyparser;
	import java.util.Set;
	import java.util.HashSet;
	import de.timoh.galaxyparser.typchecking.*;
}

@members {
	boolean isTypeName(String name) {
		for (int i = Symbols_stack.size()-1; i>=0; i--) {
			Symbols_scope scope = (Symbols_scope)Symbols_stack.get(i);
			if ( scope.types.contains(name) ) {
				return true;
			}
		}
		return false;
	}
}

@lexer::header {
  package de.timoh.galaxyparser;
}

translation_unit
scope Symbols;
@init {
  if(Main.debug)System.out.println("Debug: "+"starting parser");
  $Symbols::types = new HashSet();
}
	: external_declaration+ EOF!
	;


external_declaration
options {k=1;}
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) external_declaration-rule");
}

	: ( (declaration_specifiers)? direct_declarator declaration* '{' )=> function_definition
	| declaration
	| struct_specifier
	;

function_definition
scope Symbols; 
@init {
	if(Main.debug)System.out.println("Debug: "+"(Parser) function_definition-rule");
  $Symbols::types = new HashSet();
}
	:	(declaration_specifiers)? direct_declarator
		function_block
	;
	
function_block
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) function_block-rule");
}
	:	'{'^
		localDeclaration*
		statement*
		'}'
	;

declaration
scope {
  boolean isTypedef;
}
@init {
	if(Main.debug)System.out.println("Debug: "+"(Parser) declaration-rule");
  $declaration::isTypedef = false;
}
	: TYPEDEF^ declaration_specifiers? IDENTIFIER ';'! {$declaration::isTypedef=true;$Symbols::types.add($IDENTIFIER.text);}
	| declaration_specifiers^ init_declarator? ';'!
	| INCLUDE! STRING_LITERAL!
	;
	
localDeclaration
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) localDeclaration-rule");
}
	:	declaration_specifiers_local^ init_declarator ';'!
	;
	
declaration_specifiers_local 
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) declaration_specifiers_local-rule");
}
	:   (   CONST!
		|   type_specifier^
        )+
	;

declaration_specifiers
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) declaration_specifiers-rule");
}
	:   (   storage_class_specifier!
		|   type_specifier^
        )+
	;

init_declarator
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) init_declarator-rule");
}
	: direct_declarator ('='^ expression)?
	;

storage_class_specifier
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser)storage_class_specifier-rule");
}
	: STATIC!
	| VOID !
	| CONST!
	| NATIVE!
	;
 
type_specifier
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) type_specifier-rule");
}
	:	BOOLEAN
	|	BYTE
	|	CHAR
	|	FIXED_LIT
	|	GAMELINK
	|	COLOR
	|	INT
	|	POINT
	|	STRING
	|	ABILCMD
	|	ACTOR
	|	ACTORSCOPE
	|	BANK
	|	CAMERAINFO
	|	MARKER
	|	ORDER
	|	PLAYERGROUP
	|	REGION
	|	REVEALER
	|	SOUNDLINK
	|	TEXT
	|	TIMER
	|	TRANSMISSIONSOURCE
	|	TRIGGER
	|	UNITFILTER
	|	UNITGROUP
	|	UNITREF
	|	UNIT
	|	WAVEINFO
	|	WAVETARGET
	|	DOODAD
	|	AIFILTER
	|	SOUND
	|	WAVE
	|   type_id
	;

type_id
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) type_id-rule");
}
    :   {isTypeName(input.LT(1).getText())}? IDENTIFIER
    ;

struct_specifier
options {k=3;}
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) struct_specifier-rule");
}
	:	STRUCT^ IDENTIFIER '{' {$Symbols::types.add($IDENTIFIER.text);} struct_declaration+ '}' + ';'!
	;
 
struct_declaration
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) struct_declaration-rule");
}
	: specifier_qualifier_list^ IDENTIFIER SEMI!
	;
 
specifier_qualifier_list
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) specifier_qualifier_list-rule");
}
	: ( type_qualifier! | type_specifier^ )+
	;

type_qualifier
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) type_qualifier-rule");
}
	: CONST
	;

direct_declarator
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) direct_declarator-rule");
}
	:   
		declarator_suffix*			
			IDENTIFIER
		declarator_suffix*	    
	;

declarator_suffix
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) declarator_suffix-rule");
}
	:   '[' constant_expression ']'
    |   '['! ']'!
    |   '('! parameter_list ')'!
    |   '('! identifier_list! ')'!
    |   '('! ')'!
	;

parameter_list
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) parameter_list-rule");
}
	: parameter_declaration (',' parameter_declaration)*
	;

parameter_declaration
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) parameter_declaration-rule");
}
	: (declaration_specifiers IDENTIFIER)
	;

identifier_list
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) identifier_list-rule");
}
	: IDENTIFIER (','! IDENTIFIER)*
	; 

type_name
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) type_name-rule");
}
	: specifier_qualifier_list
	;

//expressions

argument_expression_list
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) argument_expression_list-rule");
}
	:   assignment_expression (',' assignment_expression)*
	;

additive_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) additive_expression-rule");
}
	: (multiplicative_expression) ('+'^ multiplicative_expression | '-'^ multiplicative_expression )*
	;

multiplicative_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) multiplicative_expression-rule");
}
	: unary_expression ('*'^ unary_expression | '/'^ unary_expression)*
	;

unary_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) unary_expression-rule");
}
	: postfix_expression
	| unary_operator^ unary_expression
	;

postfix_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) postfix_expression-rule");
}
	:   primary_expression
        (   '[' expression ']'
        |   '(' ')'
        |   '(' argument_expression_list ')'
        |   ('.' IDENTIFIER ('[' expression ']')*)+
        )*
	;

unary_operator
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) unary_operator-rule");
}
	: '&'!
	| '+'!
	| '-' -> NEGATION
	| '!'
	;

primary_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) primary_expression-rule");
}
	: IDENTIFIER
	| constant
	| '('! expression ')'!
	;
 
constant
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) constant-rule");
}
    :   INTEGER
    |	CHARACTER_LITERAL
	|	STRING_LITERAL
    |   FIXED
    |	TRUE
    |	FALSE
    |	NULL
    ;

/////

expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) expression-rule");
}
	: assignment_expression
	;

constant_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) constant_expression-rule");
}
	: logical_or_expression
	;

assignment_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) assignment_expression-rule");
}
	: lvalue assignment_operator^ assignment_expression
	| logical_or_expression
	;
	
lvalue
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) lvalue-rule");
}
	:	unary_expression
	;

assignment_operator
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) assignment_operator-rule");
}
	: '='^
	| '+='^
	| '-='^
	| '*='^
	| '/='^
	;

logical_or_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) logical_or_expression-rule");
}
	: logical_and_expression ('||'^ logical_and_expression)* 
	;

logical_and_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) logical_and_expression-rule");
}
	: inclusive_or_expression ('&&'^ inclusive_or_expression)*
	;

inclusive_or_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) inclusive_or_expression-rule");
}
	: exclusive_or_expression ('|'^ exclusive_or_expression)*
	;

exclusive_or_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) exclusive_or_expression-rule");
}
	: and_expression ('^'^ and_expression)*
	;

and_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) and_expression-rule");
}
	: equality_expression ('&'^ equality_expression)*
	;
equality_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) equality_expression-rule");
}
	: relational_expression (('=='^|'!='^) relational_expression)*
	;

relational_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) relational_expression-rule");
}
	: shift_expression (('<'^|'>'^|'<='^|'>='^) shift_expression)*
	;

shift_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) shift_expression-rule");
}
	: additive_expression (('<<'^|'>>'^) additive_expression)*
	;

//statements

statement
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) statement-rule");
}
	: block
	| expression_statement
	| selection_statement
	| iteration_statement
	| jump_statement
	;
	
block
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) block-rule");
}
	: '{' statement* '}'
	;

statement_list
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) statement_list-rule");
}
	: statement+
	;

expression_statement
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) expression_statement-rule");
}
	: ';'!
	| expression ';'!
	;

selection_statement
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) selection_statement-rule");
}
	: 'if'^ '('! expression ')'! statement 
	(options {k=1; backtrack=false;}:'else' statement )?	
	;

iteration_statement
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) iteration_statement-rule");
}
	: 'while'^ '('! expression ')'! statement 
	;

jump_statement
@init{
	if(Main.debug)System.out.println("Debug: "+"(Parser) jump_statement-rule");
}
	: 'continue'! ';'!
	| 'break'! ';'!
	| 'return'^ ';'
	| 'return'^ expression ';'
	;

//Lexer

//primitive types
BOOLEAN : 'bool';
BYTE : 'byte';
CHAR : 'char';
FIXED_LIT : 'fixed';
GAMELINK : 'gamelink';
COLOR : 'color';
INT : 'int';
POINT : 'point';
STRING : 'string';

//further types
ABILCMD : 'abilcmd';
ACTOR : 'actor';
ACTORSCOPE : 'actorscope';
AIFILTER : 'aifilter';
BANK : 'bank';
CAMERAINFO : 'camerainfo';
MARKER : 'marker';
ORDER : 'order';
PLAYERGROUP : 'playergroup';
REGION : 'region';
REVEALER : 'revealer';
SOUNDLINK : 'soundlink';
TEXT : 'text';
TIMER : 'timer';
TRIGGER : 'trigger';
TRANSMISSIONSOURCE : 'transmissionsource';
UNITFILTER : 'unitfilter';
UNITGROUP : 'unitgroup';
UNITREF: 'unitref';
UNIT : 'unit';
WAVEINFO : 'waveinfo';
WAVETARGET : 'wavetarget';
DOODAD : 'doodad';
SOUND : 'sound';
WAVE : 'wave';

//Other Keywords
BREAK : 'break';
CONST : 'const';
ELSE : 'else';
FALSE : 'false';
IF : 'if';
INCLUDE : 'include';
NATIVE : 'native';
NULL : 'null';
RETURN : 'return';
STATIC : 'static';
STRUCT : 'struct';
TRUE : 'true';
TYPEDEF : 'typedef';
VOID : 'void';
WHILE : 'while';
SEMI : ';';

IDENTIFIER
	:	LETTER (LETTER|'0'..'9')*
	;
	
fragment
LETTER
	:	'A'..'Z'
	|	'a'..'z'
	|	'_'
	;

CHARACTER_LITERAL
    :   '\'' ( EscapeSequence | ~('\''|'\\') ) '\''
    ;

STRING_LITERAL
    :  '"' ( EscapeSequence | ~('\\'|'"') )* '"'
    ;

INTEGER 
	: DIGIT+
	| HEX_LITERAL
	;
	
fragment
HEX_LITERAL : '0' ('x'|'X') HexDigit+;

fragment
HexDigit : ('0'..'9'|'a'..'f'|'A'..'F') ;

FIXED 
	:	DIGIT+ '.' DIGIT* 
	|	'.' DIGIT+
	;
	
fragment
DIGIT : '0'..'9';

fragment
Exponent : ('e'|'E') ('+'|'-')? ('0'..'9')+ ;

fragment
FloatTypeSuffix : ('f'|'F'|'d'|'D') ;

fragment
EscapeSequence
    :   '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\')
    |   OctalEscape
    ;

fragment
OctalEscape
    :   '\\' ('0'..'3') ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7')
    ;

fragment
UnicodeEscape
    :   '\\' 'u' HexDigit HexDigit HexDigit HexDigit
    ;

WS  :  (' '|'\r'|'\t'|'\u000C'|'\n') {$channel=HIDDEN;}
    ;

LINE_COMMENT
    : '//' ~('\n'|'\r')* '\r'? '\n' {$channel=HIDDEN;}
    ;
