tree grammar GalaxyWalker;

options {
  language = Java;
  tokenVocab = Galaxy;
  ASTLabelType = CommonTree;
}
@header {
  package de.timoh.galaxyparser;
  import de.timoh.galaxyparser.typchecking.*;
  //import java.utils.ArrayList;
}

@members {
  SymbolTable table = new SymbolTable();
  private boolean isLocal = false;
  private String structName = "";
  private String lastStructName = "";
  private Type actualReturnType = null;
}

evaluator
@init{
	if(Main.debug)System.out.println("Debug: "+"starting treeparser");
}
	: (	struct
	  | typedef
	  | var
	  | function
	  )* EOF
	;
	
var
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) var-rule");
	String tmp = "";
}
	: ^(t1=type ^('=' i1=IDENTIFIER (e1 = expression[i1.getText(),t1,i1.getLine(),i1.getCharPositionInLine()] {if(t1 != null && e1 != null){if(Type.checkAssign(t1,e1.type)){
															if(!t1.isNullable() && e1.type == Type.Null){
																System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' can\'t be nullified");
															}else{
																System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" wrong expression assigned to \'"+i1.getText()+"\'");
															}	
													   }
													   if(!table.tokenExists($i1.getText(),true)){table.put($i1.getText(),t1,true);}else{System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable with name \'"+i1.getText()+"\' allready defined");}}}
				))) 
	| ^(t2=type array_expression+ i2=IDENTIFIER) {if(!table.tokenExists($i2.getText(),true)){table.put($i2.getText(),t2.Array,true);}else{System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable with name '"+i2.getText()+"' allready defined");}}
	| ^(t3=type i3=IDENTIFIER) {if(!table.tokenExists($i3.getText(),true)){table.put($i3.getText(),t3,true);}else{System.err.println("line " +i3.getLine()+":"+i3.getCharPositionInLine()+" variable with name '"+i3.getText()+"' allready defined");}}
	;
	
function
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) function-rule");
	table.clearLocal();
	actualReturnType = null;
	boolean statementReturned = false;
}
	: (i1=IDENTIFIER (p=parameterList)? {if(table.isFunction(i1.getText())){
													System.err.println("line "+i1.getLine()+":"+i1.getCharPositionInLine()+" function with name \'"+i1.getText()+"\' is allready defined");
												}else{
													Function f = new Function(i1.getText(),Type.Void,p);
													table.putFunction(i1.getText(),f);
												}
										  }
	  ^('{'
		varLocal*
		statement*
		return_statement?
		'}'))
	| (t=type i1=IDENTIFIER (p=parameterList)? {if(table.isFunction(i1.getText())){
													System.err.println("line "+i1.getLine()+":"+i1.getCharPositionInLine()+" function with name \'"+i1.getText()+"\' is allready defined");
												}else{
													Function f = new Function(i1.getText(),t,p);
													table.putFunction(i1.getText(),f);
												}
												actualReturnType = t;
										  }
	  ^('{'
		varLocal*
		statement*
		(return_statement {statementReturned = true;})? {if(!statementReturned)System.err.println("line "+i1.getLine()+":"+i1.getCharPositionInLine()+" return statement missing for function \'"+i1.getText()+"\'");}
		'}'))
	;

varLocal
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) varLocal-rule");
	String tmp = "";
}
	: ^(t1=type ^('=' i1=IDENTIFIER (e1 = expression[i1.getText(),t1,i1.getLine(),i1.getCharPositionInLine()] {if(t1 != null && e1 != null){if(Type.checkAssign(t1,e1.type)){
															if(!t1.isNullable() && e1.type == Type.Null){
																System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' can\'t be nullified");
															}else{
																System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" wrong expression assigned to \'"+i1.getText()+"\'");
															}	
													   }
													   if(!table.tokenExists($i1.getText(),false)){table.put($i1.getText(),t1,false);}else{System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable with name \'"+i1.getText()+"\' allready defined");}}}
				))) 
	| ^(t2=type array_expression+ i2=IDENTIFIER) {if(!table.tokenExists($i2.getText(),false)){table.put($i2.getText(),t2.Array,false);}else{System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable with name '"+i2.getText()+"' allready defined");}}
	| ^(t3=type i3=IDENTIFIER) {if(!table.tokenExists($i3.getText(),false)){table.put($i3.getText(),t3,false);}else{System.err.println("line " +i3.getLine()+":"+i3.getCharPositionInLine()+" variable with name '"+i3.getText()+"' allready defined");}}
	;
	
statement
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) statement-rule");
	String tmp = "", tmp2 = "";
	Type t1 = null ,t2 = null;
	boolean array = false;
}
	: ^('=' i1=IDENTIFIER {if(table.tokenExists(i1.getText(),false)){
								t1 = table.get(i1.getText());
								if(t1.isArray()){
									System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' is an array -> index missing");
								}else{
									tmp = t1.toString();
								}
							 }else{
							 	System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable "+i1.getText()+" is not defined");
						 	 }
							}
	
	
		(t3=dot_structs[tmp]{t1 = t3;})? (e1=expression[i1.getText(),t1,i1.getLine(),i1.getCharPositionInLine()] {if(t1 != null && e1 != null){if(Type.checkAssign(t1,e1.type)){
													if(!t1.isNullable() && e1.type == Type.Null){
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" wrong expression assigned to \'"+i1.getText()+"\'");
													}	
												}}}
										  ))
	|^('=' i2=IDENTIFIER array_expression+ {if(table.tokenExists(i2.getText(),false)){
												t2 = table.get(i2.getText());
												if(t2.isArray()){
													tmp = t2.toString();
													tmp = tmp.substring(0,tmp.length()-5);
												}else{
													System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' is not an array");
												}
							 				}else{
							 					System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable "+i2.getText()+" is not defined");
						 				    }}
	
	
	
	   (t4=dot_structs[tmp]{t2 = t4;if(t2.isArray()){tmp = t2.toString();tmp = tmp.substring(0,tmp.length()-5);}else{tmp = t2.toString();}})? (e2=expression[i2.getText(),t2,i2.getLine(),i2.getCharPositionInLine()]{if(tmp != null && e2 != null){if(Type.checkAssignString(tmp,e2.type.toString())){
													
													if(!t2.isNullable() && e2.type == Type.Null){
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" wrong expression assigned to \'"+i2.getText()+"\'");
													}	
												}}}
										  ))
    |^('+=' i1=IDENTIFIER {if(table.tokenExists(i1.getText(),false)){
								t1 = table.get(i1.getText());
								if(t1.isArray()){
									System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' is an array -> index missing");
								}else{
									tmp = t1.toString();
								}
							 }else{
							 	System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable "+i1.getText()+" is not defined");
						 	 }
							}
	
	
		(t3=dot_structs[tmp]{t1 = t3;})? (e1=expression[i1.getText(),t1,i1.getLine(),i1.getCharPositionInLine()] {if(t1 != null && e1 != null){if(Type.checkPlusAssign(t1,e1.type)){
													if(!t1.isNullable() && e1.type == Type.Null){
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" wrong expression assigned to \'"+i1.getText()+"\'");
													}
														
												}}}))
	|^('+=' i2=IDENTIFIER array_expression+ {if(table.tokenExists(i2.getText(),false)){
												t2 = table.get(i2.getText());
												if(t2.isArray()){
													tmp = t2.toString();
													tmp = tmp.substring(0,tmp.length()-5);
												}else{
													System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' is not an array");
												}
							 				}else{
							 					System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable "+i2.getText()+" is not defined");
						 				    }}
	
	
	
	   (t4=dot_structs[tmp]{t2 = t4;if(t2.isArray()){tmp = t2.toString();tmp = tmp.substring(0,tmp.length()-5);}})? (e2=expression[i2.getText(),t2,i2.getLine(),i2.getCharPositionInLine()]{if(tmp != null && e2 != null){if(Type.checkPlusAssignString(tmp,e2.type.toString())){
													if(!t2.isNullable() && e2.type == Type.Null){
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" wrong expression assigned to \'"+i2.getText()+"\'");
													}	
												}}}))
    |^('-=' i1=IDENTIFIER {if(table.tokenExists(i1.getText(),false)){
								t1 = table.get(i1.getText());
								if(t1.isArray()){
									System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' is an array -> index missing");
								}else{
									tmp = t1.toString();
								}
							 }else{
							 	System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable "+i1.getText()+" is not defined");
						 	 }
							}
	
	
		(t3=dot_structs[tmp]{t1 = t3;})? (e1=expression[i1.getText(),t1,i1.getLine(),i1.getCharPositionInLine()] {if(t1 != null && e1 != null){if(Type.checkMathAssign(t1,e1.type)){
													if(!t1.isNullable() && e1.type == Type.Null){
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" wrong expression assigned to \'"+i1.getText()+"\'");
													}
														
												}}}))
	|^('-=' i2=IDENTIFIER array_expression+ {if(table.tokenExists(i2.getText(),false)){
												t2 = table.get(i2.getText());
												if(t2.isArray()){
													tmp = t2.toString();
													tmp = tmp.substring(0,tmp.length()-5);
												}else{
													System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' is not an array");
												}
							 				}else{
							 					System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable "+i2.getText()+" is not defined");
						 				    }}
	
	
	
	   (t4=dot_structs[tmp]{t2 = t4;if(t2.isArray()){tmp = t2.toString();tmp = tmp.substring(0,tmp.length()-5);}})? (e2=expression[i2.getText(),t2,i2.getLine(),i2.getCharPositionInLine()]{if(tmp != null && e2 != null){if(Type.checkMathAssignString(tmp,e2.type.toString())){
													if(!t2.isNullable() && e2.type == Type.Null){
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" wrong expression assigned to \'"+i2.getText()+"\'");
													}	
												}}}))
    |^('*=' i1=IDENTIFIER {if(table.tokenExists(i1.getText(),false)){
								t1 = table.get(i1.getText());
								if(t1.isArray()){
									System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' is an array -> index missing");
								}else{
									tmp = t1.toString();
								}
							 }else{
							 	System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable "+i1.getText()+" is not defined");
						 	 }
							}
	
	
		(t3=dot_structs[tmp]{t1 = t3;})? (e1=expression[i1.getText(),t1,i1.getLine(),i1.getCharPositionInLine()] {if(t1 != null && e1 != null){if(Type.checkMathAssign(t1,e1.type)){
													if(!t1.isNullable() && e1.type == Type.Null){
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" wrong expression assigned to \'"+i1.getText()+"\'");
													}
														
												}}}))
	|^('*=' i2=IDENTIFIER array_expression+ {if(table.tokenExists(i2.getText(),false)){
												t2 = table.get(i2.getText());
												if(t2.isArray()){
													tmp = t2.toString();
													tmp = tmp.substring(0,tmp.length()-5);
												}else{
													System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' is not an array");
												}
							 				}else{
							 					System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable "+i2.getText()+" is not defined");
						 				    }}
	
	
	
	   (t4=dot_structs[tmp]{t2 = t4;if(t2.isArray()){tmp = t2.toString();tmp = tmp.substring(0,tmp.length()-5);}})? (e2=expression[i2.getText(),t2,i2.getLine(),i2.getCharPositionInLine()]{if(tmp != null && e2 != null){if(Type.checkMathAssignString(tmp,e2.type.toString())){
													if(!t2.isNullable() && e2.type == Type.Null){
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" wrong expression assigned to \'"+i2.getText()+"\'");
													}	
												}}}))
    |^('/=' i1=IDENTIFIER {if(table.tokenExists(i1.getText(),false)){
								t1 = table.get(i1.getText());
								if(t1.isArray()){
									System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' is an array -> index missing");
								}else{
									tmp = t1.toString();
								}
							 }else{
							 	System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable "+i1.getText()+" is not defined");
						 	 }
							}
	
	
		(t3=dot_structs[tmp]{t1 = t3;})? (e1=expression[i1.getText(),t1,i1.getLine(),i1.getCharPositionInLine()] {if(t1 != null && e1 != null){if(Type.checkMathAssign(t1,e1.type)){
													if(!t1.isNullable() && e1.type == Type.Null){
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" wrong expression assigned to \'"+i1.getText()+"\'");
													}
														
												}}}))
	|^('/=' i2=IDENTIFIER array_expression+ {if(table.tokenExists(i2.getText(),false)){
												t2 = table.get(i2.getText());
												if(t2.isArray()){
													tmp = t2.toString();
													tmp = tmp.substring(0,tmp.length()-5);
												}else{
													System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' is not an array");
												}
							 				}else{
							 					System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable "+i2.getText()+" is not defined");
						 				    }}
	
	
	
	   (t4=dot_structs[tmp]{t2 = t4;if(t2.isArray()){tmp = t2.toString();tmp = tmp.substring(0,tmp.length()-5);}})? (e2=expression[i2.getText(),t2,i2.getLine(),i2.getCharPositionInLine()]{if(tmp != null && e2 != null){if(Type.checkMathAssignString(tmp,e2.type.toString())){
													if(!t2.isNullable() && e2.type == Type.Null){
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' can\'t be nullified");
													}else{
														System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" wrong expression assigned to \'"+i2.getText()+"\'");
													}	
												}}}))
	| if_statement
    | ^(w=WHILE e1=expression[null,null,0,0]	{if(e1.type != Type.Boolean){
									System.err.println("line "+w.getLine()+":"+w.getCharPositionInLine()+" while expression has to be of type bool");
								}}
    
    	'{'
    	statement*
    	return_statement?
    	'}')
    | function_call_statement
	;
	
if_statement
	:^(i=IF e1= expression[null,null,0,0] 	{if(e1 != null){if(e1.type != Type.Boolean){
									System.err.println("line "+i.getLine()+":"+i.getCharPositionInLine()+" if expression has to be of type bool");
								}}}
		'{'
		statement*	
		return_statement?
		'}'
		(ELSE (
		'{'
		 statement*
		 return_statement?
		'}'
		|
		if_statement
		)?
		)?
		)
	;
	
function_call_statement
	: function_id
	;
	
return_statement
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) return_statement-rule");
}
	: ^(r=RETURN SEMI) 	{if(actualReturnType != null){
							System.err.println("line "+r.getLine()+":"+r.getCharPositionInLine()+" return excepts arguements, no expression given");
						}}
	| ^(r=RETURN e1=expression[null,null,0,0] SEMI)	{if(actualReturnType == null){
											System.err.println("line "+r.getLine()+":"+r.getCharPositionInLine()+" return statement has to be empty");
										}else{
											if(e1.type != actualReturnType){
												System.err.println("line "+r.getLine()+":"+r.getCharPositionInLine()+" wrong expression passed to return statement");
											}
										}}
	;
	

	 
dot_structs [String structName]returns [Type t]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) dot_structs-rule");
}
	: 	(t1=dot_struct[structName] {structName = lastStructName;})+ {t = t1;}
	;
dot_struct [String s] returns [Type t]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) dot_struct-rule");
}
	:	'.' i1=IDENTIFIER {lastStructName = i1.getText();if(table.tokenExists(s+"_"+i1.getText(),true)){
							Type t1 = table.get(s+"_"+i1.getText(),true);
							if(t1.isArray()){
								System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable \'"+i1.getText()+"\' is an array -> index missing");
							}else{
								t = t1;
							}
						   }else{
								System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable "+i1.getText()+" is not defined");
						   }}
	|	'.' i2=IDENTIFIER array_expression+ {lastStructName = i2.getText();if(table.tokenExists(s+"_"+i2.getText(),true)){
							Type t1 = table.get(s+"_"+i2.getText(),true);
							if(t1.isArray()){
								t = t1;
							}else{
								System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable \'"+i2.getText()+"\' is not an array");
							}
						   }else{
								System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable "+i2.getText()+" is not defined");
						   }}
	;
parameterList returns [Parameters p]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) parameterList-rule");
	ArrayList<Type> pa = new ArrayList<Type>();
}
	: (t=parameter {pa.add(t);})+ {Type[] types = new Type[pa.size()];
								   for(int i = 0;i < types.length;i++){
								   	types[i] = pa.get(i);
								   }
								   p = new Parameters(types);
								  }
	;
	
parameter returns [Type t]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) parameter-rule");
}
	: (t2=type i=IDENTIFIER) ','? {table.put(i.getText(),t2,false);t=t2;}
	;
struct
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) struct-rule");
	table.clearLocal();
	isLocal = true;
}
	:^(STRUCT IDENTIFIER {structName = $IDENTIFIER.getText();
			  	  if(Type.isCustomType(structName)){
			  	  	System.err.println("line "+$IDENTIFIER.getLine()+":"+$IDENTIFIER.getCharPositionInLine()+structName+"is allready defined");
			      }else{
			  	  	Type.TypeCustom(structName,null);
			      }} 
			      '{'
		struct_assignment+
		'}') {isLocal = false;}
	;
	
struct_assignment
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) struct_assignment-rule");
}
	:^(t1 =type array_expression+ i1=IDENTIFIER) {if(!table.tokenExists($i1.getText(),false)){table.put($i1.getText(),t1.Array,false);table.put(structName+"_"+$i1.getText(),t1.Array,true);}else{System.err.println("line " +i1.getLine()+":"+i1.getCharPositionInLine()+" variable with name '"+i1.getText()+"' allready defined");}}
	|^(t2 =type i2=IDENTIFIER) {if(!table.tokenExists($i2.getText(),false)){table.put($i2.getText(),t2,false);table.put(structName+"_"+$i2.getText(),t2,true);}else{System.err.println("line " +i2.getLine()+":"+i2.getCharPositionInLine()+" variable with name '"+i2.getText()+"' allready defined");}}
	;
	 
array_expression
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) array_expression-rule");
}
	: l='[' e1 = expression[null,null,0,0] ']' {if(e1 != null){if(e1.type != Type.Integer)System.err.println("line " +l.getLine()+":"+(l.getCharPositionInLine()+1)+" arraydimension has to be an integer");}}
	;
	
typedef
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) typedef-rule");
}
	:^(TYPEDEF type IDENTIFIER){String t = $IDENTIFIER.getText(); 
			  					if(Type.isCustomType(t)){
			  						System.err.println(t+"is allready defined");
			  					}else{
			  						Type.TypeCustom(t,$type.t);
			  					}}
	;

expression[String s,Type t2, int l, int pos]  returns [Expr e]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) expression-rule");
}
	: ^('||' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Or(a,b);}})
	| ^('&&' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new And(a,b);}})
	| ^('==' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Rel("==",a,b);}})
	| ^('!=' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Rel("!=",a,b);}})
	| ^('<' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Rel("<",a,b);}})
	| ^('<=' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Rel("<=",a,b);}})
	| ^('>' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Rel(">",a,b);}})
	| ^('>=' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Rel(">=",a,b);}})
	| ^('+' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Arith("+",a,b);}})
	| ^('-' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Arith("-",a,b);}})
	| ^('*' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Arith("*",a,b);}})
	| ^('/' a = expression[null,null,0,0] b = expression[null,null,0,0] {if( a != null && b != null){e = new Arith("+",a,b);}})
	| ^('!' a = expression[null,null,0,0] {if( a != null){e = new Not(a);}})
	| ^(NEGATION a = expression[null,null,0,0] {if( a != null){e = new Unary("-",a);}})
	| INTEGER {e = new Constant($INTEGER.getText(),Type.Integer);}
	| FIXED {e = new Constant($FIXED.getText(),Type.Fixed);}
	| STRING_LITERAL {e = new Constant($STRING_LITERAL.getText(),Type.String);}
	| CHARACTER_LITERAL {e = new Constant($CHARACTER_LITERAL.getText(),Type.Char);}
	| TRUE {e = Constant.True;}
	| FALSE {e = Constant.False;}
	| NULL {e = Constant.Null;}
	| i = IDENTIFIER {Type ts = table.get(i.getText()); if(ts == null){System.err.println("line "+i.getLine()+":"+i.getCharPositionInLine()+" variable "+i.getText()+" is not defined");}else{e = new Constant(i.getText(),ts);}}
	| t = array_identifier[s,t2,l,pos] {e = new Constant("",t);}
	| t3=function_id {if(t3 != null){e = new Constant("void", t3);}} 
	;

array_identifier[String s,Type t2, int l, int pos] returns [Type t]
@init{
	String tmp = "";
	String tmp2 = "";
}
	:  i6=IDENTIFIER array_expression+{if(table.tokenExists(i6.getText(),false)){
						 										Type t6 = table.get(i6.getText());
																if(t6.isArray()){
																	tmp2 = t6.toString();
																	tmp2 = tmp2.substring(0,tmp2.length()-5);
																	
																}else{
																	tmp2 = t6.toString();
																}
																t = Type.getType(tmp2);
														   }else{
																System.err.println("line " +i6.getLine()+":"+i6.getCharPositionInLine()+" variable "+i6.getText()+" is not defined");
						 								   }}
		  									(t7=dot_structs[tmp2] {if(tmp != null && t7 != null && t2 != null){
		  															if(Type.checkAssignString(tmp,t7.toString())){
															   			if(!t2.isNullable() && t7 == Type.Null){
																			System.err.println("line " +l+":"+pos+" variable \'"+s+"\' can\'t be nullified");
																	    }else{
																			System.err.println("line " +l+":"+pos+" wrong expression assigned to \'"+s+"\'");
																		}	
															   		}}else{ if(t7 != null)t = t7;}})?
												
	;
	
type returns [Type t]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) type-rule");
	t = null;
}
	:	BOOLEAN {t = Type.Boolean;}
	|	BYTE {t = Type.Byte;}
	|	CHAR {t = Type.Char;}
	|	FIXED_LIT {t = Type.Fixed;}
	|	GAMELINK {t = Type.Gamelink;}
	|	COLOR {t = Type.Color;}
	|	INT {t = Type.Integer;}
	|	POINT {t = Type.Point;}
	|	STRING {t = Type.String;}
	|	ABILCMD {t = Type.Abilcmd;}
	|	ACTOR {t = Type.Actor;}
	|	ACTORSCOPE {t = Type.Actorscope;}
	|	BANK {t = Type.Bank;}
	|	CAMERAINFO {t = Type.Camerainfo;}
	|	MARKER {t = Type.Marker;}
	|	ORDER {t = Type.Order;}
	|	PLAYERGROUP {t = Type.Playergroup;}
	|	REGION {t = Type.Region;}
	|	REVEALER {t = Type.Revealer;}
	|	SOUNDLINK {t = Type.Soundlink;}
	|	TEXT {t = Type.Text;}
	|	TIMER {t = Type.Timer;}
	|	TRANSMISSIONSOURCE {t = Type.Transmissionsource;}
	|	TRIGGER {t = Type.Trigger;}
	|	UNITFILTER {t = Type.Unitfilter;}
	|	UNITGROUP {t = Type.Unitgroup;}
	|	UNITREF {t = Type.Unitref;}
	|	UNIT {t = Type.Unit;}
	|	WAVEINFO {t = Type.Waveinfo;}
	|	WAVETARGET {t = Type.Wavetarget;}
	|	DOODAD {t = Type.Doodad;}
	|	AIFILTER {t = Type.AIFilter;}
	|	SOUND {t = Type.Sound;}
	|	WAVE {t = Type.Wave;}
	|	type_id {t = $type_id.t;}
	;
	
type_id returns [Type t]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) type_id-rule");
}
	: IDENTIFIER {if(Type.isCustomType($IDENTIFIER.getText())){
 			   		t = Type.getCustomType($IDENTIFIER.getText());
 			   }else{
 			    	t = null;
 			    	
 			   }
 			   }
 ;
 
function_id returns [Type t]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) function_id-rule");
	Parameters p = null;
	Type t2 = null;
	int count = 0;
}
	: 	i=IDENTIFIER '(' ')' 	{if(table.isFunction(i.getText())){
							 		Function f = table.getFunction(i.getText());
							 		if(f.getParameters() != null){
							 			System.err.println("line "+i.getLine()+":"+i.getCharPositionInLine()+" function \'"+i.getText()+"\' excepts arguments");
							 		}else{
							 			t = table.getFunction(i.getText()).returnType;
							 	 	}
							 	 }else{
							 	 	System.err.println("line "+i.getLine()+":"+i.getCharPositionInLine()+" function \'"+i.getText()+"\' is not defined");
							 	 }	
						 		}
	| i2= IDENTIFIER {if(table.isFunction(i2.getText())){
							 Function f = table.getFunction(i2.getText());
							 t2 = f.returnType;
							 p = f.getParameters();
						}else{
							 System.err.println("line "+i2.getLine()+":"+i2.getCharPositionInLine()+" function "+i2.getText()+" is not defined");
						}	
					   }
	
		'(' (list[p,count, i2.getLine()+":"+i2.getCharPositionInLine(), i2.getText()] {count++;})+ ')'
		{t = t2;}
	;
	
	
list [Parameters p, int count, String line, String function]
@init{
	if(Main.debug)System.out.println("Debug: "+"(TreeParser) list-rule");
}
	: (e=expression[null,null,0,0] (',')?)  {if(e != null && p != null){
								if(count < p.length()){
									if(Type.checkAssign(p.get(count),e.type)){
										if(!e.type.isNullable() && p.get(count) == Type.Null){
											System.err.println("line " +line+" wrong parameters passed to function \'"+function+"\'");
										}	
								 	}
							 	}else{
							 		System.err.println("line " +line+" too many arguments passed to function \'"+function+"\'");
							 	}
								}}
	;