grammar NativeLibrary;

options {
  language = Java;
  output = AST;
  ASTLabelType=CommonTree;
}

@header{
	package de.timoh.galaxyparser;
}

@lexer::header{
	package de.timoh.galaxyparser;
}

startScan
	:	(
		native_function_decl
		|
		native_global_decl
		|
		include_decl
		)*
	;

native_function_decl
	: NATIVE^ function_qualifier IDENTIFIER function_parameters SEMI!
	| function_qualifier^ IDENTIFIER function_parameters '{'! 'return'! IDENTIFIER! '('! INTEGER! ','! IDENTIFIER! ')'! '=='! INTEGER! SEMI! '}'! //STUPID FUNCTION!! 
	;
	
function_qualifier
	: (modifier! | type_specifier^)*
	;
	
function_parameters
	: '(' ')'
	| '(' (parameters+)^ ')'
	;

parameters
	: type_specifier IDENTIFIER! (',')?
	;
	
modifier
	: STATIC
	| CONST
	| VOID
	;
native_global_decl
	: (CONST!)? type_specifier^ IDENTIFIER ('='! expression!)? SEMI!
	| type_specifier^ ('['! INTEGER! ']'!)+ IDENTIFIER SEMI!
	;
	
include_decl
	: INCLUDE! STRING_LITERAL!
	;
	
type_specifier
	:	BOOLEAN
	|	BYTE
	|	CHAR
	|	FIXED_LIT
	|	GAMELINK
	|	COLOR
	|	INT
	|	POINT
	|	STRING
	|	ABILCMD
	|	ACTOR
	|	ACTORSCOPE
	|	BANK
	|	CAMERAINFO
	|	MARKER
	|	ORDER
	|	PLAYERGROUP
	|	REGION
	|	REVEALER
	|	SOUNDLINK
	|	TEXT
	|	TIMER
	|	TRANSMISSIONSOURCE
	|	TRIGGER
	|	UNITFILTER
	|	UNITGROUP
	|	UNITREF
	|	UNIT
	|	WAVEINFO
	|	WAVETARGET
	|	DOODAD
	|	AIFILTER
	|	SOUND
	|	WAVE
	;
	
//expressions

expression
	:	constant
	|	'-' constant
	|   '('expression')'
	|	constant '>>' constant
	|	constant '<<' constant
	;

constant
	:	INTEGER
	|	FIXED
	|	STRING_LITERAL
	|	CHARACTER_LITERAL
	|	TRUE
	|	FALSE
	|	NULL
	;

//Lexer

//primitive types
BOOLEAN : 'bool';
BYTE : 'byte';
CHAR : 'char';
FIXED_LIT : 'fixed';
GAMELINK : 'gamelink';
COLOR : 'color';
INT : 'int';
POINT : 'point';
STRING : 'string';

//further types
ABILCMD : 'abilcmd';
ACTOR : 'actor';
ACTORSCOPE : 'actorscope';
AIFILTER : 'aifilter';
BANK : 'bank';
CAMERAINFO : 'camerainfo';
MARKER : 'marker';
ORDER : 'order';
PLAYERGROUP : 'playergroup';
REGION : 'region';
REVEALER : 'revealer';
SOUNDLINK : 'soundlink';
TEXT : 'text';
TIMER : 'timer';
TRIGGER : 'trigger';
TRANSMISSIONSOURCE : 'transmissionsource';
UNITFILTER : 'unitfilter';
UNITGROUP : 'unitgroup';
UNITREF: 'unitref';
UNIT : 'unit';
WAVEINFO : 'waveinfo';
WAVETARGET : 'wavetarget';
DOODAD : 'doodad';
SOUND : 'sound';
WAVE : 'wave';

//Other Keywords
CONST : 'const';
FALSE : 'false';
INCLUDE : 'include';
NATIVE : 'native';
NULL : 'null';
STATIC : 'static';
TRUE : 'true';
VOID : 'void';
SEMI : ';';

IDENTIFIER
	:	LETTER (LETTER|'0'..'9')*
	;
	
fragment
LETTER
	:	'A'..'Z'
	|	'a'..'z'
	|	'_'
	;

CHARACTER_LITERAL
    :   '\'' ( EscapeSequence | ~('\''|'\\') ) '\''
    ;

STRING_LITERAL
    :  '"' ( EscapeSequence | ~('\\'|'"') )* '"'
    ;

INTEGER 
	: DIGIT+
	| HEX_LITERAL
	;
	
fragment
HEX_LITERAL : '0' ('x'|'X') HexDigit+;

fragment
HexDigit : ('0'..'9'|'a'..'f'|'A'..'F') ;

FIXED 
	:	DIGIT+ '.' DIGIT* 
	|	'.' DIGIT+
	;
	
fragment
DIGIT : '0'..'9';

fragment
Exponent : ('e'|'E') ('+'|'-')? ('0'..'9')+ ;

fragment
FloatTypeSuffix : ('f'|'F'|'d'|'D') ;

fragment
EscapeSequence
    :   '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\')
    |   OctalEscape
    ;

fragment
OctalEscape
    :   '\\' ('0'..'3') ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7')
    ;

fragment
UnicodeEscape
    :   '\\' 'u' HexDigit HexDigit HexDigit HexDigit
    ;

WS  :  (' '|'\r'|'\t'|'\u000C'|'\n') {$channel=HIDDEN;}
    ;

LINE_COMMENT
    : '//' ~('\n'|'\r')* '\r'? '\n' {$channel=HIDDEN;}
    ;